import numpy as np

from sklearn.datasets import load_files
from sklearn.feature_extraction.text import CountVectorizer
from sklearn.feature_extraction.text import TfidfTransformer
from sklearn.externals import joblib

from sklearn.linear_model import SGDClassifier

def run_classifier(cls_model
						, cls_countvect
						, cls_transformer
						, cls_databunch
						, filename):

	classifier      = joblib.load(cls_model)
	count_vect      = joblib.load(cls_countvect)
	transformer     = joblib.load(cls_transformer)
	data_bunch      = joblib.load(cls_databunch)
	
	in_file = open(filename, 'r', encoding='utf-8', errors='ignore')
	docs_new = [in_file.read()]
		
	X_new_counts = count_vect.transform(docs_new)
	X_new_tfidf	 = transformer.transform(X_new_counts)
		
	predicted = classifier.predict(X_new_tfidf)
	print(predicted)
		
	predicted_proba = classifier.predict_proba(X_new_tfidf)
	print(predicted_proba)
	
	return data_bunch.target_names[predicted[0]]

def build_classifier(dir_name, test_dir_name):

	classes = ['k1', 'k2']

	train_data = load_files(dir_name, classes)

	print(str(len(train_data.data)) + " dokumenata")
	print("Klase: ")
	print(train_data.target_names)
	print("Sadrzaj: ")
	print(train_data.data)
	print('-----------------------------------------')
	
	count_vect   = CountVectorizer(decode_error = 'ignore')
	#count_vect   = CountVectorizer(decode_error = 'ignore'
	#						, ngram_range=(1, 2), analyzer='word')
	#count_vect   = CountVectorizer(decode_error = 'ignore'
	#									, ngram_range=(9, 9)
	#									, analyzer='char')
	train_counts = count_vect.fit_transform(train_data.data)
	
	features = count_vect.get_feature_names()
	print(features)
	print(train_counts)
	print('-----------------------------------------')
	
	tfidf_transformer = TfidfTransformer()
	train_tfidf = tfidf_transformer.fit_transform(train_counts)
	print(train_tfidf)
	print('-----------------------------------------')
	
	classifier = SGDClassifier(max_iter=1000, loss='log').fit(train_tfidf, train_data.target)
	
	joblib.dump(classifier		  , './sgd_classifier.pkl' , compress=9)
	joblib.dump(tfidf_transformer , './sgd_transformer.pkl', compress=9) 
	joblib.dump(count_vect		  , './sgd_countvect.pkl'  , compress=9) 
	joblib.dump(train_data        , './sgd_traindata.pkl'  , compress=9) 
		
	print('Training is done')
	print('-----------------------------------------')

	test_data       = load_files(test_dir_name, classes)
	test_counts     = count_vect.transform(test_data.data)
	test_tfidf      = tfidf_transformer.transform(test_counts)
	
	predicted = classifier.predict(test_tfidf)
	print(test_data.target)
	print(predicted)
	
	predicted_proba = classifier.predict_proba(test_tfidf)
	print(predicted_proba)
	
	acc = round(np.mean(predicted == test_data.target) * 100, 2)
	print("Accuracy = " + str(acc))
	
#build_classifier('./A', './A')
print(run_classifier('./sgd_classifier.pkl'
							, './sgd_countvect.pkl'
							, './sgd_transformer.pkl'
							, './sgd_traindata.pkl'
							, './A/k1/1.txt'))

